@@ -2,6 +2,7 @@
 Tests for the API (logged in user)
 """
 
+import os
 import json
 
 import pytest
@@ -96,7 +97,6 @@ def test_update_job(auth_client):
 
     original_info = auth_client.get("api/jobs/1").json
     assert original_info["status"].lower() == "finished"
-
     response = auth_client.put(
         "api/jobs/1",
         data=json.dumps({"status": "submitted"}),
@@ -110,9 +110,54 @@ def test_update_job(auth_client):
     assert response.status_code == 201
 
     new_info = auth_client.get("api/jobs/1").json
+
     assert new_info["status"] == "submitted"
 
 
+def test_update_project(auth_client):
+    """Check put method of api/jobs/{job_ID}"""
+
+    csrf_token = auth_client[1]
+    auth_client = auth_client[0]
+    original_info = auth_client.get("api/projects/1").json
+    assert original_info["description"] is None
+
+    response = auth_client.put(
+        "api/projects/1",
+        data=json.dumps({"description": "testing update"}),
+        headers={
+            "Accept": "application/json",
+            "Content-Type": "application/json",
+            "X-CSRF-TOKEN": csrf_token,
+        },
+    )
+
+    assert response.status_code == 201
+
+    new_info = auth_client.get("api/projects/1").json
+    assert new_info["description"] == "testing update"
+
+
+def test_add_project(auth_client):
+    csrf_token = auth_client[1]
+    auth_client = auth_client[0]
+
+    project = {"name": "added_project"}
+
+    response = auth_client.post(
+        "api/projects",
+        data=json.dumps(project),
+        content_type="application/json",
+        headers={"X-CSRF-TOKEN": csrf_token},
+    )
+
+    assert response.status_code == 201
+
+    from seamm_dashboard import datastore
+
+    assert os.path.exists(os.path.join(datastore, "projects", "added_project"))
+
+
 def test_get_users(auth_client):
     """Check get method of api/users on authenticated client"""
 
@@ -132,7 +177,7 @@ def test_get_projects(auth_client):
 
     assert response.status_code == 200
 
-    assert len(response.json) == 1
+    assert len(response.json) == 2
 
 
 def test_get_project(auth_client):
